#! /usr/bin/env python

import sys
from PyQt4 import QtCore, QtGui


def debug(s):
    sys.stderr.write(s + '\n')

files = [("foo", 12),
         ("bar", 13),
         ("baz", 72)]


class FooBarModel(QtCore.QAbstractItemModel):
    def __init__(self, files, parent=None):
        QtCore.QAbstractItemModel.__init__(self, parent)

        self.files = files
        self.num_columns = 2

    def rowCount(self, parent):
        debug("*** rowCount(self, parent=%s)" % (parent,))
        debug("parent.internalPointer() = %s" % (parent.internalPointer(),))
        debug("parent.isValid(): %s" % parent.isValid())

        if parent.isValid():
            res = 0
        else:
            res = len(self.files)

        debug("-> return %s" % res)
        return res

    def columnCount(self, parent):
        debug("*** columnCount(self, parent=%s)" % (parent,))
        debug("parent.internalPointer() = %s" % (parent.internalPointer(),))

        if parent.isValid():
            res = 0
        else:
            res = self.num_columns

        debug("-> return %s" % res)
        return res

    def index(self, row, column, parent):
        debug("*** index(self, row=%s, column=%s, parent=%s)" % \
              (row, column, parent))
        debug("parent.internalPointer() = %s" % (parent.internalPointer(),))

        # Only the root element has childs
        if parent.isValid():
            return QtCore.QModelIndex()

        if row < 0 or column < 0 \
           or row >= len(self.files) or column >= self.num_columns:
            return QtCore.QModelIndex()

        return self.createIndex(row, column, self.files[row])

    def data(self, index, role):
        debug("*** data(self, index=%s, role=%s)" % (index, role))
        debug("index.internalPointer() = %s" % (index.internalPointer(),))

        if not index.isValid():
            return QtCore.QVariant()

        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()

        item = index.internalPointer()

        if index.column() == 0:
            data = item[0]
        elif index.column() == 1:
            data = item[1]
        else:
            assert False, index.column()
        
        return QtCore.QVariant(data)

    def flags(self, index):
        debug("*** flags(self, index=%s)" % (index,))

        if not index.isValid():
            return QtCore.Qt.ItemIsEnabled
        else:
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable

    def headerData(self, section, orientation, role):
        debug("*** headerData(self, section=%s, orientation=%s, role=%s)" \
              % (section, orientation, role))

        data = map(QtCore.QVariant, [self.tr("File"), self.tr("Size")])

        if orientation == QtCore.Qt.Horizontal \
               and role == QtCore.Qt.DisplayRole:
            res = data[section]
        else:
            res = QtCore.QVariant()

        debug("-> return %s (str: '%s')" % (repr(res), res.toString()))
        return res

    def parent(self, index):
        debug("*** parent(self, index=%s)" % (index,))

        return QtCore.QModelIndex()


def test():
    model = FooBarModel(files)
    view = QtGui.QTreeView()
    view.setModel(model)

    return view, model


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)

    # With this line, everything works
#    view, model = test()

    # But with this one, no luck
    view = test()[0]

    view.show()

    sys.exit(app.exec_())
